#ifndef SIRIL
#define SIRIL
#define EXIT 99
#include <stdio.h>
#include <fitsio.h>
#include <fftw3.h>
#define max(a,b) (((a) > (b)) ? (a) : (b))
#define min(a,b) (((a) < (b)) ? (a) : (b))
#ifndef GTK
#define GTK
#include <gtk/gtk.h>
#endif

#define SEQSOF 64
#define ST_IDLE 0x00
#define ST_BUSY 0x01
#define ST_CANCEL 0x02

typedef unsigned char BYTE;
typedef short WORD;

#define USEFLI 0x01
#define USEDARK 0x02
#define USEOFFS 0x04
#define USEFLAT 0x08

#define SEQREGD 0x01
#define SEQBYNAME 0x2
#define SEQBYQUAL 0x4
#define SEQREVERSE 0x8
#define SEQSHOWEXC 0x10


#define MAXGENLENGTH 256

#define ZOOMMAX 5
#define ZOOMMIN (-5)
#define NOZOOM	0

#define NOREMAP 0
#define REMAP 1

#define TYPEUNDEF 5
#define TYPEFITS 4 
#define TYPEPNG 0
#define TYPEJPG 1
#define TYPEBMP 2
#define TYPEAVI 6

#define USE_DARK 0x01
#define USE_FLAT 0x02
#define USE_OFFSET 0x04

#define OD_NULL 	0
#define OD_FLAT 	1
#define OD_DARK 	2
#define OD_OFFSET 	3
#define OD_CWD 		4
#define OD_OPEN 	5

#define CD_NULL 	0
#define CD_INCALL 	1
#define CD_EXCALL 	2

#define BW_LAYER 	0
#define RLAYER 	0
#define GLAYER 	1
#define BLAYER 	2
#define RGB_LAYER 	3

#define MAXVPORT 	4
#define BW_VPORT 	0
#define RED_VPORT 	0
#define GREEN_VPORT 	1
#define BLUE_VPORT 	2
#define RGB_VPORT 	3

// defines for copyfits actions 

#define CP_INIT		0x01	// initialize data array with 0s
#define CP_ALLOC	0x02	// reallocs data array
#define CP_COPYA	0x04	// copy data array content
#define CP_FORMAT	0x08	// copy size and bitpix info
#define CP_EXTRACT	0x10	// extract a 16bit plane from a 48 bit fit
#define CP_EXPAND	0x20	// expands a 16bit fits to a 48bit one. 

#define CONVALL 0x01
#define CONVBMP 0x02
#define CONVPNG 0x04
#define CONVJPG 0x08
#define CONVAVI 0x10
#define CONVPNM 0x20
#define CONV1X3 0x40
#define CONV3X1 0x80
#define CONV1X1 0x100
#define CONVUFL 0x200

#define SIZESHORT 16
#define SIZEBYTE 8

typedef struct imdata imgdata;
struct imdata {
	int filenum;		/* file index in the sequence mars9.fit = 9*/
	int qual_rank;		/* quality rank */
	int shiftx;
   	int shifty;
	int quality;
	gboolean incl;
	};

typedef struct sequ sequence;
struct sequ {
	gboolean active;
	int fixed;
	char name[MAXGENLENGTH];
	char *names;
	char darkname[MAXGENLENGTH];
	char darkprefix[MAXGENLENGTH];
	char flatname[MAXGENLENGTH];
	char flatprefix[MAXGENLENGTH];
	char offsetname[MAXGENLENGTH];
	char offsetprefix[MAXGENLENGTH];
	char selprefix[MAXGENLENGTH];
	char ppprefix[MAXGENLENGTH];
	int selnum;
	int number;
	char fli;
	int imagenumber;
	int current;
	unsigned status;
	int type;
	int beg;
	int end;
	imgdata *imgparam;
	};

typedef struct ffit fits;
struct ffit{
	unsigned int rx;
	unsigned int ry;
	unsigned int lo;
	unsigned int hi;
	int bitpix;
	int naxis;
	long naxes[4];
	unsigned short maxi; 
	unsigned short mini; 
	unsigned short min[3]; 
	unsigned short max[3]; 
	fitsfile *fptr;
	void *cdata;
	short *pdata[4];
	short *data;
	};

typedef struct cominf cominfo;
struct cominf {
	guchar *graybuf[3];	// one B/W display buffer per viewport (R,G,B)
	guchar *rgbbuf;		// one rgb display buffer
	GtkWidget *vport[4];	// one drawingarea per layer, one rgb drawingarea
	int cvport;		// current viewport
	gboolean drawn, drawing, autolevel, rgb;
	int rx;
	int ry;
	int leveldrag;
	int reglayer;
	int preprostatus;
	gint zoom;
	gint upx; 
	gint upy;
	gint dnx;
	gint dny;
	gint llx; // selection width
	gint lly; // selection height 
	gint lx;  // up left corner coord x
	gint ly;  // up left corner coord y
	gint cly;  // up left corner coord y, top-bottom flipped
	gchar busy;
	guchar confirm;
	guchar beg;
	guchar end;
	char wd[256];
	char status[256];
	char format[256];
	char genname[256];
	char formname[256];
	char initfile[256];
	int imagetype;
	float darkopt;
	sequence seq;
} ;

typedef struct comm command;
struct comm {
	char name[32];
	int nbarg;
	char usage[128];
	int (* process)(int);
	};

#include "proto.h"
#ifdef MAIN
cominfo com;
GtkWidget *main_window;
GtkWidget *rgb_window;
GtkWidget *OpenDial;
GtkWidget *registration_window;
GtkWidget *stack_window;
GtkWidget *medstackwindow;
GtkWidget *about_window;
GtkWidget *seqproc_window;
GtkWidget *setworkingdir_dialog;
GtkWidget *confirm_dialog;
GdkGC *gc;
fits gfit;
fits wfit[6];
#else
extern cominfo com;
extern GtkWidget *main_window;
extern GtkWidget *rgb_window;
extern GtkWidget *OpenDial;
extern GtkWidget *registration_window;
extern GtkWidget *stack_window;
extern GtkWidget *medstackwindow;
extern GtkWidget *about_window;
extern GtkWidget *seqproc_window;
extern GtkWidget *setworkingdir_dialog;
extern GtkWidget *confirm_dialog;
extern GdkGC *gc;
extern fits gfit;
extern fits wfit[6];
#endif
#endif /*SIRIL */
